@@ -1,5 +1,5 @@
 /* Creation of autonomous subprocesses.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -61,6 +61,10 @@
 
 #endif
 
+#ifndef HAVE_ENVIRON_DECL
+extern char **environ;
+#endif
+
 #ifndef STDIN_FILENO
 # define STDIN_FILENO 0
 #endif
@@ -117,6 +121,7 @@ nonintr_open (const char *pathname, int oflag, mode_t mode)
 int
 execute (const char *progname,
 	 const char *prog_path, char **prog_argv,
+	 bool ignore_sigpipe,
 	 bool null_stdin, bool null_stdout, bool null_stderr,
 	 bool slave_process, bool exit_on_error)
 {
@@ -306,7 +311,7 @@ execute (const char *progname,
       unblock_fatal_signals ();
     }
 
-  return wait_subprocess (child, progname, null_stderr,
+  return wait_subprocess (child, progname, ignore_sigpipe, null_stderr,
 			  slave_process, exit_on_error);
 
 #endif
