@@ -36,6 +36,7 @@
 #include "sh-quote.h"
 #include "safe-read.h"
 #include "xalloc.h"
+#include "xallocsa.h"
 #include "error.h"
 #include "gettext.h"
 
@@ -122,7 +123,7 @@ compile_java_class (const char * const *java_sources,
 	  command_length += 1 + shell_quote_length (java_sources[i]);
 	command_length += 1;
 
-	command = (char *) alloca (command_length);
+	command = (char *) xallocsa (command_length);
 	p = command;
 	/* Don't shell_quote $JAVAC, because it may consist of a command
 	   and options.  */
@@ -162,9 +163,11 @@ compile_java_class (const char * const *java_sources,
 	argv[2] = command;
 	argv[3] = NULL;
 	exitstatus = execute (javac, "/bin/sh", argv, false, false, false,
-			      true, true);
+			      false, true, true);
 	err = (exitstatus != 0);
 
+	freesa (command);
+
 	/* Reset CLASSPATH.  */
 	reset_classpath (old_classpath);
 
@@ -236,7 +239,8 @@ compile_java_class (const char * const *java_sources,
 
 	    /* Remove zombie process from process list, and retrieve exit
 	       status.  */
-	    exitstatus = wait_subprocess (child, "gcj", true, true, false);
+	    exitstatus =
+	      wait_subprocess (child, "gcj", false, true, true, false);
 	    if (exitstatus != 0)
 	      gcj_present = false;
 	  }
@@ -263,7 +267,7 @@ compile_java_class (const char * const *java_sources,
 	argc =
 	  2 + (optimize ? 1 : 0) + (debug ? 1 : 0) + (directory != NULL ? 2 : 0)
 	  + java_sources_count;
-	argv = (char **) alloca ((argc + 1) * sizeof (char *));
+	argv = (char **) xallocsa ((argc + 1) * sizeof (char *));
 
 	argp = argv;
 	*argp++ = "gcj";
@@ -291,10 +295,12 @@ compile_java_class (const char * const *java_sources,
 	    free (command);
 	  }
 
-	exitstatus = execute ("gcj", "gcj", argv, false, false, false, true,
-			      true);
+	exitstatus = execute ("gcj", "gcj", argv, false, false, false, false,
+			      true, true);
 	err = (exitstatus != 0);
 
+	freesa (argv);
+
 	/* Reset CLASSPATH.  */
 	reset_classpath (old_classpath);
 
@@ -314,8 +320,8 @@ compile_java_class (const char * const *java_sources,
 
 	argv[0] = "javac";
 	argv[1] = NULL;
-	exitstatus = execute ("javac", "javac", argv, false, true, true, true,
-			      false);
+	exitstatus = execute ("javac", "javac", argv, false, false, true, true,
+			      true, false);
 	javac_present = (exitstatus == 0 || exitstatus == 1 || exitstatus == 2);
 	javac_tested = true;
       }
@@ -339,7 +345,7 @@ compile_java_class (const char * const *java_sources,
 	argc =
 	  1 + (optimize ? 1 : 0) + (debug ? 1 : 0) + (directory != NULL ? 2 : 0)
 	  + java_sources_count;
-	argv = (char **) alloca ((argc + 1) * sizeof (char *));
+	argv = (char **) xallocsa ((argc + 1) * sizeof (char *));
 
 	argp = argv;
 	*argp++ = "javac";
@@ -367,9 +373,11 @@ compile_java_class (const char * const *java_sources,
 	  }
 
 	exitstatus = execute ("javac", "javac", argv, false, false, false,
-			      true, true);
+			      false, true, true);
 	err = (exitstatus != 0);
 
+	freesa (argv);
+
 	/* Reset CLASSPATH.  */
 	reset_classpath (old_classpath);
 
@@ -389,8 +397,8 @@ compile_java_class (const char * const *java_sources,
 
 	argv[0] = "jikes";
 	argv[1] = NULL;
-	exitstatus = execute ("jikes", "jikes", argv, false, true, true, true,
-			      false);
+	exitstatus = execute ("jikes", "jikes", argv, false, false, true, true,
+			      true, false);
 	jikes_present = (exitstatus == 0 || exitstatus == 1);
 	jikes_tested = true;
       }
@@ -416,7 +424,7 @@ compile_java_class (const char * const *java_sources,
 	argc =
 	  1 + (optimize ? 1 : 0) + (debug ? 1 : 0) + (directory != NULL ? 2 : 0)
 	  + java_sources_count;
-	argv = (char **) alloca ((argc + 1) * sizeof (char *));
+	argv = (char **) xallocsa ((argc + 1) * sizeof (char *));
 
 	argp = argv;
 	*argp++ = "jikes";
@@ -444,9 +452,11 @@ compile_java_class (const char * const *java_sources,
 	  }
 
 	exitstatus = execute ("jikes", "jikes", argv, false, false, false,
-			      true, true);
+			      false, true, true);
 	err = (exitstatus != 0);
 
+	freesa (argv);
+
 	/* Reset CLASSPATH.  */
 	reset_classpath (old_classpath);
 
