@@ -1,5 +1,5 @@
 /* Reading file lists.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,21 @@
 
 #include "str-list.h"
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /* Read list of filenames from a file.
    One filename per line.  Lines starting with # and whitespace lines are
    ignored.  Trailing whitespace is removed.  */
 extern string_list_ty *read_names_from_file (const char *file_name);
 
+
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* _FILE_LIST_H */
