@@ -1,5 +1,5 @@
 /* xgettext Emacs Lisp backend.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2001-2002.
 
@@ -28,10 +28,10 @@
 #include <string.h>
 
 #include "message.h"
-#include "x-elisp.h"
 #include "xgettext.h"
+#include "x-elisp.h"
 #include "error.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "hash.h"
 #include "c-ctype.h"
@@ -116,6 +116,13 @@ init_keywords ()
     }
 }
 
+void
+init_flag_table_elisp ()
+{
+  xgettext_record_flag ("_:1:pass-elisp-format");
+  xgettext_record_flag ("format:1:elisp-format");
+}
+
 
 /* ======================== Reading of characters.  ======================== */
 
@@ -339,7 +346,7 @@ comment_add (int c)
 {
   if (buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen++] = c;
@@ -354,7 +361,7 @@ comment_line_end (size_t chars_to_remove)
     --buflen;
   if (chars_to_remove == 0 && buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
@@ -424,6 +431,9 @@ string_of_object (const struct object *op)
   return str;
 }
 
+/* Context lookup table.  */
+static flag_context_list_table_ty *flag_context_list_table;
+
 /* Returns the character represented by an escape sequence.  */
 #define IGNORABLE_ESCAPE (EOF - 1)
 static int
@@ -526,7 +536,7 @@ do_getc_escaped (int c, bool in_string)
     case '^':
       c = do_getc ();
       if (c == EOF)
-        return EOF;
+	return EOF;
       if (c == '\\')
 	{
 	  c = do_getc ();
@@ -609,7 +619,8 @@ do_getc_escaped (int c, bool in_string)
    'first_in_list' and 'new_backquote_flag' are used for reading old
    backquote syntax and new backquote syntax.  */
 static void
-read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
+read_object (struct object *op, bool first_in_list, bool new_backquote_flag,
+	     flag_context_ty outer_context)
 {
   for (;;)
     {
@@ -634,6 +645,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	case '(':
 	  {
 	    int arg = 0;		/* Current argument number.  */
+	    flag_context_list_iterator_ty context_iter;
 	    int argnum1 = 0;		/* First string position.  */
 	    int argnum2 = 0;		/* Plural string position.  */
 	    message_ty *plural_mp = NULL;	/* Remember the msgid.  */
@@ -641,8 +653,18 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	    for (;; arg++)
 	      {
 		struct object inner;
+		flag_context_ty inner_context;
 
-		read_object (&inner, arg == 0, new_backquote_flag);
+		if (arg == 0)
+		  inner_context = null_context;
+		else
+		  inner_context =
+		    inherited_context (outer_context,
+				       flag_context_list_iterator_advance (
+					 &context_iter));
+
+		read_object (&inner, arg == 0, new_backquote_flag,
+			     inner_context);
 
 		/* Recognize end of list.  */
 		if (inner.type == t_listclose)
@@ -660,58 +682,64 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 		if (inner.type == t_eof)
 		  break;
 
-		/* No need to bother if we extract all strings anyway.  */
-		if (!extract_all)
+		if (arg == 0)
 		  {
-		    if (arg == 0)
+		    /* This is the function position.  */
+		    if (inner.type == t_symbol)
 		      {
-			/* This is the function position.  */
-			if (inner.type == t_symbol)
+			char *symbol_name = string_of_object (&inner);
+			void *keyword_value;
+
+			if (find_entry (&keywords,
+					symbol_name, strlen (symbol_name),
+					&keyword_value)
+			    == 0)
 			  {
-			    char *symbol_name = string_of_object (&inner);
-			    void *keyword_value;
-
-			    if (find_entry (&keywords,
-					    symbol_name, strlen (symbol_name),
-					    &keyword_value)
-				== 0)
-			      {
-				argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-				argnum2 = (int) (long) keyword_value >> 10;
-			      }
-
-			    free (symbol_name);
+			    argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
+			    argnum2 = (int) (long) keyword_value >> 10;
 			  }
+
+			context_iter =
+			  flag_context_list_iterator (
+			    flag_context_list_table_lookup (
+			      flag_context_list_table,
+			      symbol_name, strlen (symbol_name)));
+
+			free (symbol_name);
 		      }
 		    else
+		      context_iter = null_context_list_iterator;
+		  }
+		else
+		  {
+		    /* These are the argument positions.
+		       Extract a string if we have reached the right
+		       argument position.  */
+		    if (arg == argnum1)
 		      {
-			/* These are the argument positions.
-			   Extract a string if we have reached the right
-			   argument position.  */
-			if (arg == argnum1)
+			if (inner.type == t_string)
 			  {
-			    if (inner.type == t_string)
-			      {
-				lex_pos_ty pos;
-				message_ty *mp;
+			    lex_pos_ty pos;
+			    message_ty *mp;
 
-				pos.file_name = logical_file_name;
-				pos.line_number = inner.line_number_at_start;
-				mp = remember_a_message (mlp, string_of_object (&inner), &pos);
-				if (argnum2 > 0)
-				  plural_mp = mp;
-			      }
+			    pos.file_name = logical_file_name;
+			    pos.line_number = inner.line_number_at_start;
+			    mp = remember_a_message (mlp, string_of_object (&inner),
+						     inner_context, &pos);
+			    if (argnum2 > 0)
+			      plural_mp = mp;
 			  }
-			else if (arg == argnum2)
+		      }
+		    else if (arg == argnum2)
+		      {
+			if (inner.type == t_string && plural_mp != NULL)
 			  {
-			    if (inner.type == t_string && plural_mp != NULL)
-			      {
-				lex_pos_ty pos;
+			    lex_pos_ty pos;
 
-				pos.file_name = logical_file_name;
-				pos.line_number = inner.line_number_at_start;
-				remember_a_message_plural (plural_mp, string_of_object (&inner), &pos);
-			      }
+			    pos.file_name = logical_file_name;
+			    pos.line_number = inner.line_number_at_start;
+			    remember_a_message_plural (plural_mp, string_of_object (&inner),
+						       inner_context, &pos);
 			  }
 		      }
 		  }
@@ -736,7 +764,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	      {
 		struct object inner;
 
-		read_object (&inner, false, new_backquote_flag);
+		read_object (&inner, false, new_backquote_flag, null_context);
 
 		/* Recognize end of vector.  */
 		if (inner.type == t_vectorclose)
@@ -770,7 +798,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	  {
 	    struct object inner;
 
-	    read_object (&inner, false, new_backquote_flag);
+	    read_object (&inner, false, new_backquote_flag, null_context);
 
 	    /* Dots and EOF are not allowed here.  But be tolerant.  */
 
@@ -787,7 +815,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	  {
 	    struct object inner;
 
-	    read_object (&inner, false, true);
+	    read_object (&inner, false, true, null_context);
 
 	    /* Dots and EOF are not allowed here.  But be tolerant.  */
 
@@ -811,7 +839,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	  {
 	    struct object inner;
 
-	    read_object (&inner, false, false);
+	    read_object (&inner, false, false, null_context);
 
 	    /* Dots and EOF are not allowed here.  But be tolerant.  */
 
@@ -836,7 +864,11 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 		if (c != ';')
 		  all_semicolons = false;
 		if (!all_semicolons)
-		  comment_add (c);
+		  {
+		    /* We skip all leading white space, but not EOLs.  */
+		    if (!(buflen == 0 && (c == ' ' || c == '\t')))
+		      comment_add (c);
+		  }
 	      }
 	    comment_line_end (0);
 	    continue;
@@ -888,7 +920,8 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 
 		pos.file_name = logical_file_name;
 		pos.line_number = op->line_number_at_start;
-		remember_a_message (mlp, string_of_object (op), &pos);
+		remember_a_message (mlp, string_of_object (op),
+				    null_context, &pos);
 	      }
 	    last_non_comment_line = line_number;
 	    return;
@@ -941,7 +974,8 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 		    {
 		      struct object inner;
 
-		      read_object (&inner, false, new_backquote_flag);
+		      read_object (&inner, false, new_backquote_flag,
+				   null_context);
 
 		      /* Recognize end of vector.  */
 		      if (inner.type == t_vectorclose)
@@ -976,7 +1010,8 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	      /* Read a bit vector.  */
 	      {
 		struct object length;
-		read_object (&length, first_in_list, new_backquote_flag);
+		read_object (&length, first_in_list, new_backquote_flag,
+			     null_context);
 		/* Dots and EOF are not allowed here.
 		   But be tolerant.  */
 		free_object (&length);
@@ -985,7 +1020,8 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	      if (c == '"')
 		{
 		  struct object string;
-		  read_object (&string, first_in_list, new_backquote_flag);
+		  read_object (&string, first_in_list, new_backquote_flag,
+			       null_context);
 		  free_object (&string);
 		}
 	      else
@@ -1002,7 +1038,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	      {
 		struct object inner;
 		do_ungetc (c);
-		read_object (&inner, false, new_backquote_flag);
+		read_object (&inner, false, new_backquote_flag, null_context);
 		/* Dots and EOF are not allowed here.
 		   But be tolerant.  */
 		free_object (&inner);
@@ -1043,7 +1079,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	    case 'S': case 's': /* XEmacs only */
 	      {
 		struct object inner;
-		read_object (&inner, false, new_backquote_flag);
+		read_object (&inner, false, new_backquote_flag, null_context);
 		/* Dots and EOF are not allowed here.
 		   But be tolerant.  */
 		free_object (&inner);
@@ -1069,7 +1105,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 		}
 	      if (c == '=')
 		{
-		  read_object (op, false, new_backquote_flag);
+		  read_object (op, false, new_backquote_flag, outer_context);
 		  last_non_comment_line = line_number;
 		  return;
 		}
@@ -1137,7 +1173,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 	      /* Simply assume every feature expression is true.  */
 	      {
 		struct object inner;
-		read_object (&inner, false, new_backquote_flag);
+		read_object (&inner, false, new_backquote_flag, null_context);
 		/* Dots and EOF are not allowed here.
 		   But be tolerant.  */
 		free_object (&inner);
@@ -1202,6 +1238,7 @@ read_object (struct object *op, bool first_in_list, bool new_backquote_flag)
 void
 extract_elisp (FILE *f,
 	       const char *real_filename, const char *logical_filename,
+	       flag_context_list_table_ty *flag_table,
 	       msgdomain_list_ty *mdlp)
 {
   mlp = mdlp->item[0]->messages;
@@ -1214,6 +1251,8 @@ extract_elisp (FILE *f,
   last_comment_line = -1;
   last_non_comment_line = -1;
 
+  flag_context_list_table = flag_table;
+
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When read_object returns
@@ -1222,7 +1261,7 @@ extract_elisp (FILE *f,
     {
       struct object toplevel_object;
 
-      read_object (&toplevel_object, false, false);
+      read_object (&toplevel_object, false, false, null_context);
 
       if (toplevel_object.type == t_eof)
 	break;
