@@ -1,5 +1,5 @@
 /* xgettext librep backend.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,13 @@
   { "jl",        "librep"     },					\
 
 #define SCANNERS_LIBREP \
-  { "librep",     extract_librep, &formatstring_librep },		\
+  { "librep",		extract_librep,					  \
+			&flag_table_librep, &formatstring_librep, NULL }, \
 
 /* Scan a librep file and add its translatable strings to mdlp.  */
 extern void extract_librep (FILE *fp, const char *real_filename,
 			    const char *logical_filename,
+			    flag_context_list_table_ty *flag_table,
 			    msgdomain_list_ty *mdlp);
 
 
@@ -33,3 +35,5 @@ extern void extract_librep (FILE *fp, const char *real_filename,
 
 extern void x_librep_extract_all (void);
 extern void x_librep_keyword (const char *name);
+
+extern void init_flag_table_librep (void);
