@@ -1,5 +1,5 @@
 /* xgettext Perl backend.
-   Copyright (C) 2002-2003 Free Software Foundation, Inc.
+   Copyright (C) 2002-2004 Free Software Foundation, Inc.
 
    This file was written by Guido Flohr <guido@imperia.net>, 2002-2003.
 
@@ -155,7 +155,7 @@ init_flag_table_perl ()
   xgettext_record_flag ("gettext_noop:1:pass-perl-brace-format");
   xgettext_record_flag ("printf:1:perl-format"); /* argument 1 or 2 ?? */
   xgettext_record_flag ("sprintf:1:perl-format");
-#if 0 
+#if 0
   xgettext_record_flag ("__:1:pass-perl-format");
   xgettext_record_flag ("__:1:pass-perl-brace-format");
   xgettext_record_flag ("%__:1:pass-perl-format");
@@ -253,6 +253,7 @@ phase1_getc ()
   return linebuf[linepos++];
 }
 
+/* Supports only one pushback character.  */
 static void
 phase1_ungetc (int c)
 {
@@ -479,6 +480,7 @@ phase2_getc ()
   return c;
 }
 
+/* Supports only one pushback character.  */
 static void
 phase2_ungetc (int c)
 {
@@ -2457,6 +2459,13 @@ x_perl_prelex (message_list_ty *mlp, token_ty *tp)
 	      tp->type = token_type_dereference;
 	      return;
 	    }
+	  else if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
+	    {
+	      /* One of the -X (filetest) functions.  We play safe
+		 and accept all alphabetical characters here.  */
+	      tp->type = token_type_other;
+	      return;
+	    }
 	  phase1_ungetc (c);
 	  tp->type = token_type_other;
 	  prefer_division_over_regexp = false;
