@@ -35,6 +35,9 @@
 
 #define _(s) gettext(s)
 
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
+
 /* The relevant parts of the Smalltalk syntax are:
 
      stringliteral ::= string | stringconst | symconst
@@ -96,6 +99,7 @@ phase1_getc ()
   return c;
 }
 
+/* Supports only one pushback character.  */
 static void
 phase1_ungetc (int c)
 {
@@ -178,8 +182,7 @@ struct token_ty
 
 /* 2. Combine characters into tokens.  Discard comments and whitespace.  */
 
-/* Maximum used guaranteed to be < 2.  */
-static token_ty phase2_pushback[2];
+static token_ty phase2_pushback[1];
 static int phase2_pushback_length;
 
 static void
@@ -434,11 +437,16 @@ phase2_get (token_ty *tp)
     }
 }
 
+/* Supports only one pushback token.  */
 static void
 phase2_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase2_pushback[phase2_pushback_length++] = *tp;
+    {
+      if (phase2_pushback_length == SIZEOF (phase2_pushback))
+	abort ();
+      phase2_pushback[phase2_pushback_length++] = *tp;
+    }
 }
 
 
